{-|
Module      : Main
Description : Textmining Collaborative Platform
Copyright   : (c) CNRS, 2023-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

@Gargantext@: search, map, share
-}


module Main where

import Conduit
import Data.ByteString qualified as BS
import Data.ByteString.Lazy qualified as BSL
import Data.Conduit.Combinators qualified as Conduit
import Data.Conduit.List qualified as CL
import Data.Csv qualified as Csv
import Data.Csv.Conduit qualified as CsvC
import OpenAlex qualified as OA
import OpenAlex.Types qualified as OA
import Options.Applicative.Simple
import Protolude

data Options = Options
  { filter :: Maybe OA.Filter
  , search :: Maybe OA.Search }

data ToCSVOptions = ToCSVOptions
  { options :: Options
  , output  :: FilePath }

main :: IO ()
main = do
  let filterHelp = help "Filter, for example: display_name.search:einstein , see https://docs.openalex.org/how-to-use-the-api/get-lists-of-entities/filter-entity-lists"
  let commonOptions =
        Options <$>
         optional (strOption (long "filter")) <*>
         optional (strOption (long "search"))

  let toCsvOptions =
        ToCSVOptions <$>
          commonOptions <*>
          strOption (long "output")

  (opts, runCmd) <-
    simpleOptions "0.1.0.0"
                  "OpenAlex"
                  "OpenAlex command line"
                  (pure ()) $ do
      addCommand "concepts"
        "Fetch OpenAlex concepts (https://docs.openalex.org/api-entities/concepts/concept-object)"
        fetchConcepts
        commonOptions
      addCommand "to-csv"
        "Fetch OpenAlex works (https://docs.openalex.org/api-entities/works/work-object) and save to CSV (format compatible with Gargantext Corpus CSV)"
        toCSVC
        toCsvOptions
      addCommand "works"
        "Fetch OpenAlex works (https://docs.openalex.org/api-entities/works/work-object)"
        fetchWorks
        commonOptions
      addCommand "works-c"
        "Fetch OpenAlex works (https://docs.openalex.org/api-entities/works/work-object) with conduit"
        fetchWorksC
        commonOptions

  runCmd ()


fetchConcepts :: Options -> () -> IO ()
fetchConcepts Options { .. } _ = do
  ec <- OA.fetchConcepts (Just 1) (Just 1) (Just "*") filter search
  case ec of
    Left err -> putText $ "error: " <> show err
    Right c -> do
      putText $ show c

fetchWorks :: Options -> () -> IO ()
fetchWorks Options { .. } _ = do
  ew <- OA.fetchWorks (Just 1) (Just 1) (Just "*") filter search
  case ew of
    Left err -> putText $ "error: " <> show err
    Right w -> do
      putText $ show w

fetchWorksC :: Options -> () -> IO ()
fetchWorksC Options { .. } _ = do
  eWorksC <- OA.fetchWorksC Nothing filter search
  case eWorksC of
    Left err -> putText $ "error: " <> show err
    Right (mCount, c) -> do
      putText $ "Count: " <> show mCount
      _ <- runConduit $ c
                       .| takeC 3
                       .| mapM_C (\w@(OA.Work { .. }) -> do
                                     liftIO $ do
                                       putText $ show id <> " :: " <> show display_name
                                       putText abstract_reconstructed
                                       putText "-----"
                       )
      pure ()

toCSVC :: ToCSVOptions -> () -> IO ()
toCSVC ToCSVOptions { options = Options { .. }, .. } _ = do
  eWorksC <- OA.fetchWorksC Nothing filter search
  case eWorksC of
    Left err -> putText $ "error: " <> show err
    Right (mCount, c) -> do
      putText $ "Count: " <> show mCount
      _d <- sourceToList $
           c
        -- .| mapC (\(OA.Work { .. }) -> (id, fromMaybe "" display_name, abstract_reconstructed))
        -- .| takeC 3
        .| toNamedCsv Csv.defaultEncodeOptions
        .| mapM_C (BS.appendFile output)
      -- BSL.writeFile output $ Csv.encodeDefaultOrderedByName d
      pure ()


toNamedCsv :: (Monad m, Csv.DefaultOrdered a, Csv.ToNamedRecord a)
           => Csv.EncodeOptions
           -> ConduitT a BS.ByteString m ()
toNamedCsv opts = {-# SCC toCsv_p #-} CL.map $ BSL.toStrict . Csv.encodeDefaultOrderedByNameWith opts . pure
